
HTTP_TIME_OUT = 30
API_HOST = "http://cloudserver.deepmd.net"


ALI_STS_ENDPOINT = 'http://oss-cn-shenzhen.aliyuncs.com'
ALI_STS_BUCKET_NAME = 'dpcloudserver'
ALI_OSS_BUCKET_URL = 'https://dpcloudserver.oss-cn-shenzhen.aliyuncs.com/'
# ALI_OSS_BUCKET_URL = 'https://dpcloudserver.oss-cn-shenzhen.aliyuncs.com/
